##Siligites script=group
##movealloc=name

# Module input
##SCR=crs
##MNT=raster
##Couche_Points=vector point
##Fonction=selection Tobler (h & min);Tobler hors chemin (h & min);Tobler modifiée par Marquez-Perez et al (h & min);Irmischer-Clarke (homme, sur un sentier) (h & min);Irmischer-Clarke (homme, hors sentier) (h & min);Irmischer-Clarke (femme, sur un chemin) (h & min);Irmischer-Clarke (femme, hors sentier) (h & min);Uriarte Gonzalez (h & min);Marin Arroyo (h & min);Alberti modifiée pour les excursions pastorales de recherche de nourriture (h & min);Garmy, Kaddouri, Rozenblat et Schneider (h & min);Rees (h & min);Kondo-Seino (h & min);Tripcevich (h & min);Véhicules à roues (h & min);Dépense énergétique (None);Bellavia (None);Eastman (None);Pandolf et al (Watt);Pandolf et al avec correction de decente de pente (Watt);Minetti et al (J/(kg*m));Herzog (J/(kg*m));Van Leusen (Watt);Llobera-Sluckin (kJ/m);Ardigo et al (J/(kg*m));Hare (cal/km) ;
##Interval_Isolignes=number 0
##Mouvement=enum literal 16;8;4 ;
##Vitesse=number 1.2
##Poids_du_Marcheur=number 70
##Poids_Transporte=number 0
##Cognitive_Slope=selection TRUE;FALSE ;

# Module output
##Isolignes=output vector
##Polygones=output vector
##Raster_Division=output raster
##showplots


print("Verification de la presence et chargement des librairies necessaires")

# Verification for R libraries
packages <- c("sp","sf","raster","progress","movecost")
installed_packages <- packages %in% rownames(installed.packages())
if (any(installed_packages == FALSE)) {
  install.packages(packages[!installed_packages])
}
invisible(lapply(packages, library, character.only = TRUE))

# Loading libraries
library(sp)
library(sf)
library(movecost)
library(progress)
library(raster)


print("Transformation des couches en entrees")

# Recovery of the SCR starting point and transformation of QGIS layers into objects usable by R
origin_crs <- SCR
p <- as_Spatial(Couche_Points)
studyplot_sp <- raster(MNT@file@name)


print("Verification des valeurs du raster")

# Verification for the raster's values
summary(studyplot_sp)  
any(is.na(studyplot_sp)) 
any(studyplot_sp == Inf)  
studyplot_sp[is.na(studyplot_sp)] <- 0  
studyplot_sp[studyplot_sp == Inf] <- max(studyplot_sp, na.rm = TRUE)  


print("Recuperation des valeurs renseignees")

# Definion of the function to choose a response from a choice list
get_string_value <- function(val, string_map) {
    string_map[val + 1] # +1 because R indexing starts from 1
}

# Getting the fonction name
functions_names <- c("Tobler (h & min)", "Tobler hors chemin (h & min)", "Tobler modifiée par Marquez-Perez et al (h & min)", "Irmischer-Clarke (homme, sur un sentier) (h & min)", "Irmischer-Clarke (homme, hors sentier) (h & min)", "Irmischer-Clarke (femme, sur un chemin) (h & min)", "Irmischer-Clarke (femme, hors sentier) (h & min)", "Uriarte Gonzalez (h & min)", "Marin Arroyo (h & min)", "Alberti modifiée pour les excursions pastorales de recherche de nourriture (h & min)", "Garmy, Kaddouri, Rozenblat et Schneider (h & min)", "Rees (h & min)", "Kondo-Seino (h & min)", "Tripcevich (h & min)", "Véhicules à roues (h & min)", "Dépense énergétique (None)", "Bellavia (None)", "Eastman (None)", "Pandolf et al (Watt)", "Pandolf et al avec correction de decente de pente (Watt)", "Minetti et al (J/(kg*m))", "Herzog (J/(kg*m))", "Van Leusen (Watt)", "Llobera-Sluckin (kJ/m)", "Ardigo et al (J/(kg*m))", "Hare (cal/km)")
function_map <- c("t", "tofp", "mp", "icmonp", "icmoffp", "icfonp", "icfoffp", "ug", "ma", "alb", "gkrs", "r", "ks", "trp", "wcs", "ree", "b", "e", "p", "pcf", "m", "hrz", "vl", "ls", "a", "h")
Fonction <- get_string_value(Fonction, functions_names)
i <- match(Fonction, functions_names)
Fonction <- function_map[i]

Bool <- c("FALSE","TRUE")
Cognitive_Slope <- get_string_value(Cognitive_Slope, Bool)
Cognitive_Slope <- as.logical(Cognitive_Slope)

Mouvement<-as.integer(Mouvement)

l_fonction_t <- list("t", "tofp", "mp", "icmonp", "icmoffp", "icfonp", "icfoffp", "ug", "ma", "alb", "gkrs", "r", "ks", "trp")

if (Fonction %in% l_fonction_t){
  if (Interval_Isolignes==0){
    iso <- NULL
  } else{
    iso <- Interval_Isolignes
  }
} else {
  iso <- NULL
}

print("Lancement de l'outil...")

# Launching function movealloc
tryCatch({
  r<-movealloc(
    dtm=studyplot_sp, 
    origin=p, 
    funct=Fonction, 
    breaks=iso, 
    isolines=TRUE,
    W=Poids_du_Marcheur, 
    L=Poids_Transporte, 
    V=Vitesse,
    cogn.slp=Cognitive_Slope)
},
error = function(e) {
  message(str(e))
})

print("Preparation des fichiers de sortie...")

l_fonction_abs <- list("wcs", "ree", "b", "e")
l_fonction_watt <- list("p", "pcf", "vl")
l_fonction_J <- list("m", "hrz", "a")

if (!is.null(r$cost.allocation.raster)) {
acc= r$cost.allocation.raster
raster <- acc
# Définition du SCR pour les éléments de sortie s'il ce n'est pas déjà le cas
if (is.na(crs(raster))) {
  crs(raster) <- SCR 
}
Raster_Division=raster
}

if (!is.null(r$isolines)) {
  a1=r$isolines
  sf_object = st_as_sf(a1)
  # Définition du SCR pour les éléments de sortie s'il ce n'est pas déjà le cas
  if (is.na(st_crs(sf_object))) {
    sf_object <- st_set_crs(sf_object, SCR) 
  }

  if (Fonction %in% l_fonction_t){
    colnames(sf_object)[colnames(sf_object) == 'level'] <- 'temps_h'
    sf_object$temps_h <- as.numeric(sf_object$temps_h)
    cout <- as.numeric(sf_object$temps_h)
    heures <- floor(cout)
    jours <- floor(heures/24)
    heures_2 <- floor(heures-24*jours)
    minutes <- floor((cout-heures)*60)
    hm <- paste(heures,"h",minutes,"min")
    jhm <- paste(jours,"j",heures_2,"h",minutes,"min")
    sf_object$temps_hm <- hm
    sf_object$temps_jhm <- jhm
  } else{
    colnames(sf_object)[colnames(sf_object) == 'level'] <- 'cout_energie'
    sf_object$cout_energie <- as.numeric(sf_object$cout_energie)
    if (Fonction %in% l_fonction_abs){
      sf_object$unite <- "Unité abstraite"
    } else if (Fonction %in% l_fonction_watt){
      sf_object$unite <- "Watts"
    } else if (Fonction %in% l_fonction_J){
      sf_object$unite <- "J/(kg*m)"
    } else if (Fonction == "ls"){
      sf_object$unite <- "kJ/m"
    } else if (Fonction == "h"){
      sf_object$unite <- "cal/km"
    }
  }

  sf_object$fonction <- Fonction
  sf_object$mouvements <- Mouvement
  sf_object$vitesse <- Vitesse
  sf_object$poids_marcheur <- Poids_du_Marcheur
  sf_object$poids_transport <- Poids_Transporte
  Isolignes=sf_object
}

if (!is.null(r$alloc.boundaries)) {
  a2<-r$alloc.boundaries
  sf_object2 = st_as_sf(a2)
  # Définition du SCR pour les éléments de sortie s'il ce n'est pas déjà le cas
  if (is.na(st_crs(sf_object2))) {
    sf_object2 <- st_set_crs(sf_object2, SCR) 
  }

  sf_object2$fonction <- Fonction
  sf_object2$mouvements <- Mouvement
  sf_object2$vitesse <- Vitesse
  sf_object2$poids_marcheur <- Poids_du_Marcheur
  sf_object2$poids_transport <- Poids_Transporte
  Polygones=sf_object2
}

print("Algorithme termine")

traceback()